package option_either.common.exception;

/**
 * Diese Exception sollte geworfen werden,
 * wenn in einem switch oder einer 
 * if-else-if-Kaskade ein Code-Zweig
 * erreicht wird, der bei korrektem
 * Code und korrekten Daten niemals
 * durchlaufen werden darf.
 * 
 * Diese Exception ist ein Hinweis
 * auf einem Programmfehler.
 * 
 * @author Heiner K&uuml;cker
 */
public final class UnreachableCodeException 
extends RuntimeException 
{
	/**
	 * serial version id generated by eclipse
	 */
	private static final long serialVersionUID = 2936078825627069816L;

	/**
	 * Konstruktor.
	 */
	public UnreachableCodeException()
	{
		super( "Programmfehler: unreachable code, diese Codestelle sollte niemals durchlaufen werden." );
	}

	/**
	 * Konstruktor.
	 * 
     * @param  message the detail message (which is saved for later retrieval
     *         by the {@link #getMessage()} method).
     *         
	 * @see RuntimeException#RuntimeException(String)
	 */
	public UnreachableCodeException(
			final String message )
	{
		super( message );
	}

	/**
	 * Konstruktor.
	 * 
     * @param  cause the cause (which is saved for later retrieval by the
     *         {@link #getCause()} method).  (A <tt>null</tt> value is
     *         permitted, and indicates that the cause is nonexistent or
     *         unknown.)
     *         
	 * @see RuntimeException#RuntimeException(Throwable)
	 */
	public UnreachableCodeException(
			final Throwable cause )
	{
		super( cause );
	}

	/**
	 * Konstruktor.
	 * 
     * @param  message the detail message (which is saved for later retrieval
     *         by the {@link #getMessage()} method).
     * @param  cause the cause (which is saved for later retrieval by the
     *         {@link #getCause()} method).  (A <tt>null</tt> value is
     *         permitted, and indicates that the cause is nonexistent or
     *         unknown.)
	 * @see RuntimeException#RuntimeException(String, Throwable)
	 */
	public UnreachableCodeException(
			final String message , 
			final Throwable cause )
	{
		super( 
		        message , 
		        cause );
	}

}
