package option_either.example.suche.instance_of;



/**
 * Klasse zum Aufrufen einer Suche
 * als Beispiel fr die Nutzung
 * des Option-/Either-Style in Java.
 * 
 * @author Heiner K&uuml;cker
 */
public final class InstanceOfSucheClient
{
	/**
	 * Beispiel-Code, nicht zum Ausfhren vorgesehen.
	 * 
	 * @param args unused
	 */
	public static void main(
			final String[] args )
	{
		// gekapseltes Ergebnis der Suche
		final InstanceOfSucheResult sucheResult =
				// Ausfhren einer Suche
		        InstanceOfSucheService.search(
		                args[ 0 ] );

		// das gewnschte Ergebnis, welches aus dem Ergebnis der 
		// Suche erzeugt werden soll, ist eine HTML-Seite
		final String htmlStr =
				// Instanziieren der Either(Entweder)-Klasse 
				// zum Behandeln des Ergebnisses der Suche
				sucheResult.new Either<String>()
				{
					@Override
					public String found(
							final String sucheResultStr )
					{
						// Daten gefunden
						return renderDataHtml(
								sucheResultStr );
					}

					@Override
					public String notFound()
					{
					    // keine Daten gefunden
						return renderNotFoundHtml( 
								args[ 0 ] );
					}

					@Override
					public String validationError(
							final String msgStr )
					{
						// Validierungsfehler aufgetreten
						// (zum Beispiel nicht korrekte Benutzer-Eingabe)
						return renderValidationErrorHtml(
								msgStr );
					}

					@Override
					public String error(
							final String msgStr )
					{
						// Fehler aufgetreten
						return renderErrorHtml(
								msgStr );
					}
				}.
				// am Ende auszufhrende Methode, welche die Entscheidung 
				// ber die Art des Ergebnisses trifft
				execute();

		responseWriterAppend(
				htmlStr );
	}

	/**
	 * Beispielhafte Methode zum Behandeln
	 * eines korrekten gefundenen Such-Ergebnisses,
	 * nicht zum Ausfhren vorgesehen.
	 * 
	 * @param sucheResultStr
	 * @return erzeugter HTML-String
	 */
	private static String renderDataHtml(
			String sucheResultStr )
	{
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Beispielhafte Methode zum Behandeln
	 * eines nicht gefundenen Such-Ergebnisses,
	 * nicht zum Ausfhren vorgesehen.
	 * 
	 * @param sucheResultStr
     * @return erzeugter HTML-String
	 */
	private static String renderNotFoundHtml(
			String sucheResultStr )
	{
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Beispielhafte Methode zum Behandeln
	 * eines aufgetretenen validierungsfehlers,
	 * nicht zum Ausfhren vorgesehen.
	 * 
	 * @param sucheResultStr
     * @return erzeugter HTML-String
	 */
	private static String renderValidationErrorHtml(
			String sucheResultStr )
	{
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Beispielhafte Methode zum Behandeln
	 * eines aufgetretenen technischen Fehlers,
	 * nicht zum Ausfhren vorgesehen.
	 * 
	 * @param sucheResultStr
     * @return erzeugter HTML-String
	 */
	private static String renderErrorHtml(
			String sucheResultStr )
	{
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Beispielhafte Methode zum Weitergeben
	 * des aus dem Suchergebnis erzeugten
	 * HTML-Ergebnisses,
	 * nicht zum Ausfhren vorgesehen.
	 * 
	 * @param htmlStr
	 */
	private static void responseWriterAppend(
			final String htmlStr )
	{
		// TODO Auto-generated method stub
	}

}
