package option_either.example.suche.instance_of;

import static org.junit.Assert.*;
import junit.framework.Assert;


import org.junit.Test;

/**
 * JUnit4-Testcase fr {@link InstanceOfSucheResult}.
 * 
 * @author Heiner K&uuml;cker
 */
public class InstanceOfSucheResultTest
{

    /**
     * Test method for {@link InstanceOfSucheResult#found}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testFound()
    {
        final InstanceOfSucheResult sucheResult =
                InstanceOfSucheResult.found(
                        //sucheResultStr
                        "suche war ok" );
        
        final InstanceOfSucheResult.Either<String> either =
                sucheResult.new Either<String>()
                {
                    @Override
                    public String found(
                            final String sucheResultStr )
                    {
                        Assert.assertEquals(
                                //expected
                                "suche war ok" ,
                                //actual
                                sucheResultStr );

                        return "method found called";
                    }
        
                    @Override
                    public String notFound()
                    {
                        fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String validationError(
                            final String msgStr )
                    {
                        fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String error(
                            final String msgStr )
                    {
                        fail( "wrong call" );
                        return null;
                    }
                };

        final String actual =
                either.execute();

        Assert.assertEquals(
                "method found called" ,
                actual );
    }

    /**
     * Test method for {@link InstanceOfSucheResult#notFound}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testNotFound()
    {
        final InstanceOfSucheResult sucheResult =
                InstanceOfSucheResult.notFound();
        
        final InstanceOfSucheResult.Either<String> either =
                sucheResult.new Either<String>()
                {
                    @Override
                    public String found(
                            final String sucheResultStr )
                    {
                        fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String notFound()
                    {
                        return "method notFound called";
                    }
        
                    @Override
                    public String validationError(
                            final String msgStr )
                    {
                        fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String error(
                            final String msgStr )
                    {
                        fail( "wrong call" );
                        return null;
                    }
                };

        final String actual =
                either.execute();

        Assert.assertEquals(
                "method notFound called" ,
                actual );
    }

    /**
     * Test method for {@link InstanceOfSucheResult#validationError}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testValidationError()
    {
        final InstanceOfSucheResult sucheResult =
                InstanceOfSucheResult.validationError(
                        //msgStr
                        "Parameter hat falsches Format" );
        
        final InstanceOfSucheResult.Either<String> either =
                sucheResult.new Either<String>()
                {
                    @Override
                    public String found(
                            final String sucheResultStr )
                    {
                        fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String notFound()
                    {
                        fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String validationError(
                            final String msgStr )
                    {
                        Assert.assertEquals(
                                //expected
                                "Parameter hat falsches Format" ,
                                //actual
                                msgStr );

                        return "method validationError called";
                    }
        
                    @Override
                    public String error(
                            final String msgStr )
                    {
                        fail( "wrong call" );
                        return null;
                    }
                };

        final String actual =
                either.execute();

        Assert.assertEquals(
                "method validationError called" ,
                actual );
    }

    /**
     * Test method for {@link InstanceOfSucheResult#error}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testError()
    {
        final InstanceOfSucheResult sucheResult =
                InstanceOfSucheResult.error(
                        //msgStr
                        "Suche fehlgeschlagen" );
        
        final InstanceOfSucheResult.Either<String> either =
                sucheResult.new Either<String>()
                {
                    @Override
                    public String found(
                            final String sucheResultStr )
                    {
                        fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String notFound()
                    {
                        fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String validationError(
                            final String msgStr )
                    {
                        fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String error(
                            final String msgStr )
                    {
                        Assert.assertEquals(
                                //expected
                                "Suche fehlgeschlagen" ,
                                //actual
                                msgStr );

                        return "method error called";
                    }
                };

        final String actual =
                either.execute();

        Assert.assertEquals(
                "method error called" ,
                actual );
    }

}
