package option_either.example.suche.instance_of;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import option_either.common.exception.ValidationException;

/**
 * Klasse zum Ausfhren einer Suche
 * als Beispiel fr die Nutzung
 * des Option-/Either-Style in Java.
 * 
 * @author Heiner K&uuml;cker
 */
public final class InstanceOfSucheService
{
    /**
     * Logger
     */
    // TODO private static final Logger LOGGER = Logger.getLogger( SucheService.class );

    /**
     * Konstruktor private,
     * Instanziieren nicht erlaubt.
     */
    private InstanceOfSucheService()
    {
        super();
    }

    /**
     * Daten Suchen.
     * 
     * @param suchStr Suchkriterium als String
     * @return Ergebnisdaten im Option-/Either-Style
     */
    public static InstanceOfSucheResult search(
            final String suchStr )
    {
        try
        {
            return 
            		innerSuche(
                            suchStr );
        }
        catch ( final ValidationException vexc )
        {
            // TODO LOGGER.error(
        	System.out.println(
                    "Fehler in Suche " +
                    vexc );

            return InstanceOfSucheResult.validationError(
                    // Ausgabe Fehlermeldung ohne StackTrace
                    //msgStr
                    vexc.getMessage() );
        }
        catch ( final Exception exc )
        {
            // TODO LOGGER.error(
        	System.out.println(
                    "Fehler in Suche " +
                    exc );

            final String stackTraceStr =
                    stackTraceToStr(
                            exc );

            return InstanceOfSucheResult.error(
                    // Ausgabe Fehlermeldung mit StackTrace
                    //msgStr
                    exc.getMessage() + "\n" +
                    stackTraceStr );
        }
    }

    /**
     * @param exc
     * @return StackTrace als String
     */
    private static String stackTraceToStr(
            final Exception exc )
    {
        final ByteArrayOutputStream baos =
                new ByteArrayOutputStream();

        final PrintStream printStream =
                new PrintStream(
                        baos );

        exc.printStackTrace(
                printStream );

        printStream.flush();
        printStream.close();

        final String stackTraceStr =
                new String(
                        baos.toByteArray() );

        return stackTraceStr;
    }

    /**
     * @param suchStr
     * @return {@link InstanceOfSucheResult}-Objekt
     * @throws ValidationException
     */
    private static InstanceOfSucheResult innerSuche(
            final String suchStr )
    throws ValidationException
    {
    	final String sucheErgebnisFakeStr =
    			// TODO hier Suche mocken
    			"mock";

        if ( sucheErgebnisFakeStr == null )
        {
            return InstanceOfSucheResult.notFound();
        }

        return
                InstanceOfSucheResult.found(
                        sucheErgebnisFakeStr );
    }

}
