package option_either.generator.definition;

import option_either.generator.definition.clazz.AbstractTypeVariable;

/**
 * Definition einer zu generierenden
 * Klasse nach dem Either-Muster.
 * 
 * @author Heiner K&uuml;cker
 */
public final class EitherDefinition
{
    /**
     * Kommentar/Erluterung der Either-Definition,
     * dies wird der Klassen-Kommentar
     * der generierten Either-Klasse.
     */
    public final String description;

    /**
     * Name der Either-Definition,
     * dies wird der Klassen-Name
     * der generierten Either-Klasse.
     */
    public final String name;

    /**
     * Option, ob der Aufruf des inneren Switch im Konstruktor oder 
     * in einer zu generierenden execute-Methode erfolgen soll.
     */
    public final SwitchExecutionMethod switchExecutionMethod;

    /**
     * Type-Parameter.
     */
    public final AbstractTypeVariable[] typeParameterArr;
    
    /**
     * Die verschiedenen Varianten(Ausprgungen)
     * dieser Either-Definition.
     */
    public final EitherVariant[] variantArr;

    /**
     * Konstruktor.
     * 
     * @param description Kommentar/Erluterung der Either-Definition
     * @param name Name der Either-Definition
     * @param switchExecutionMethod Option, ob der Aufruf des inneren Switch im Konstruktor oder in einer zu generierenden execute-Methode erfolgen soll
     * @param typeParameterArr Type-Parameter-Array
     * @param variantArr Varianten(Ausprgungen)
     */
    public EitherDefinition(
            final String description ,
            final String name ,
            final SwitchExecutionMethod switchExecutionMethod ,
            final AbstractTypeVariable[] typeParameterArr ,
            final EitherVariant... variantArr )
    {
        this.description = description;
        this.name = name;
        this.switchExecutionMethod = switchExecutionMethod;
        this.typeParameterArr = typeParameterArr;
        this.variantArr = variantArr;
    }
    
}
