package option_either.generator.definition;

import option_either.generator.definition.clazz.AbstractTypeParameter;

/**
 * Eine Variante(Ausprgung) einer zu 
 * generierenden Klasse nach dem 
 * Either-Muster.
 * 
 * @author Heiner K&uuml;cker
 */
public final class EitherVariant
{
    /**
     * Kommentar/Erluterung der Either-Variante,
     * dies wird der Klassen-Kommentar
     * der generierten inneren Either-Klasse.
     */
    public final String description;

    /**
     * Name der Either-Variante,
     * dies wird der Klassen-Name
     * der generierten inneren
     * Either-Klasse.
     */
    public final String name;
    
    /**
     * Type-Parameter.
     */
    public final AbstractTypeParameter[] typeParameterArr;
    
    /**
     * Die Parameter/Member dieser Either-Variante
     */
    public final Param[] paramArr;

    /**
     * Konstruktor.
     * 
     * @param description Kommentar/Erluterung der Either-Variante
     * @param name Name der Either-Variante
     * @param typeParameterArr Type-Parameter-Array
     * @param paramArr Parameter/Member
     */
    public EitherVariant(
            final String description ,
            final String name ,
            final AbstractTypeParameter[] typeParameterArr ,
            final Param... paramArr )
    {
        this.description = description;
        this.name = name;
        this.typeParameterArr = typeParameterArr;
        this.paramArr = paramArr;
    }

}
