package option_either.generator.definition;

import option_either.generator.definition.clazz.ClassWithTypeParams;

/**
 * Ein Parameter/Member einer Either-Variante.
 * 
 * @author Heiner K&uuml;cker
 */
public final class Param
{
    /**
     * Kommentar/Erluterung des Parameter/Member,
     * dies wird der Kommentar
     * der generierten Parameter/Member.
     */
    public final String description;

    /**
     * Type.
     */
    public final ClassWithTypeParams clazz;

    /**
     * Name
     */
    public final String name;

    /**
     * ob vararg (drei Punkte)
     */
    public final boolean isVararg;

    /**
     * Konstruktor.
     * 
     * @param description Kommentar/Erluterung Member/Parameter
     * @param clazz Type
     * @param name Name
     * @param isVararg ob vararg (drei Punkte)
     */
    private Param(
            final String description ,
            final ClassWithTypeParams clazz ,
            final String name ,
            final boolean isVararg )
    {
        this.description = description;
        this.clazz = clazz;
        this.name = name;
        this.isVararg = isVararg;
    }
    
    /**
     * Factory-Methode kein vararg.
     * 
     * @param description Kommentar/Erluterung Member/Parameter
     * @param clazz Type
     * @param name Name
     * @return {@link Param}-Objekt
     */
    public static Param param(
            final String description ,
            final ClassWithTypeParams clazz ,
            final String name )
    {
        return new Param(
                description ,
                clazz ,
                name ,
                //isVararg
                false );
    }

    /**
     * Factory-Methode vararg.
     * 
     * @param description Kommentar/Erluterung Member/Parameter
     * @param clazz Type
     * @param name Name
     * @return {@link Param}-Objekt
     */
    public static Param varargParam(
            final String description ,
            final ClassWithTypeParams clazz ,
            final String name )
    {
        return new Param(
                description ,
                clazz ,
                name ,
                //isVararg
                true );
    }

}
