package option_either.generator.definition.clazz;

import option_either.util.SeparatedStrBuff;

/**
 * Klassendefinition mit Type-Parametern.
 * 
 * @author Heiner K&uuml;cker
 */
public final class ClassWithTypeParams
{
    /**
     * Klasse.
     */
    public final Class<?> clazz;
    
    /**
     * Type-Parameter.
     */
    public final AbstractTypeParameter[] typeParamArr;

    /**
     * Konstruktor.
     * 
     * @param clazz Klasse
     * @param typeParamArr Type-Parameter
     */
    public ClassWithTypeParams(
            final Class<?> clazz ,
            final AbstractTypeParameter... typeParamArr )
    {
        this.clazz = clazz;
        this.typeParamArr = typeParamArr;
    }

    /**
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString()
    {
        if ( this.typeParamArr == null ||
                this.typeParamArr.length == 0 )
        {
            return this.clazz.getCanonicalName();
        }

        final SeparatedStrBuff buff = new SeparatedStrBuff( ", " );

        for ( final AbstractTypeParameter typeParameter : this.typeParamArr )
        {
            buff.add( typeParameter.toString() );
        }

        return
                this.clazz.getCanonicalName() +
                "<" +
                buff +
                ">";
    }

}
