package option_either.generator.example.constructor_switch;

import java.io.File;
import java.io.IOException;
import java.util.List;

import option_either.common.exception.ValidationException;
import option_either.generator.EitherGenerator;
import option_either.generator.definition.EitherDefinition;
import option_either.generator.definition.EitherVariant;
import option_either.generator.definition.Param;
import option_either.generator.definition.SwitchExecutionMethod;
import option_either.generator.definition.clazz.AbstractTypeVariable;
import option_either.generator.definition.clazz.ClassWithTypeParams;
import option_either.generator.definition.clazz.TypeVariable;

/**
 * Allgemeine Code-Generator-Klasse fr Option-/Either-Muster.
 * <br/>
 * Variante mit Switch-Aufruf im Konstruktor.
 * 
 * @author Heiner K&uuml;cker
 */
public final class ExampleEitherWithConstructorGenerator
{
    /**
     * src-Verzeichnis im aktuellen Eclipse-Projekt als String.
     */
    private static final String PROJECT_SRC_FOLDER_STR =
                    System.getProperty(
                            "user.dir" ) +
                    File.separator +
                    "src";

    /**
     * Package der zu generierenden Java-Klassen.
     */
    private static final String DESTINATION_PACKAGE_STR =
            "option_either.generator.example.constructor_switch.generated";

    /**
     * Start Generierung.
     * 
     * @param args unused
     * @throws IOException rethrown
     */
    public static void main( 
            final String[] args )
    throws IOException
    {
        final AbstractTypeVariable[] typeParameterArr =
                new AbstractTypeVariable[] {
                    new TypeVariable(
                            "Type-Parameter, Element-Typ der Ergebnisliste der Suche" ,
                            "E" ) };

        final EitherDefinition definition =
                new EitherDefinition(
                        //description
                        "Beispiel for Either-Code-Generator" ,
                        //name
                        "ExampleEitherWithConstructor" ,
                        SwitchExecutionMethod.CONSTRUCTOR ,
                        typeParameterArr ,
                        //variantArr
                        new EitherVariant(
                                //description
                                "erfolgreiche Suche" ,
                                //name
                                "Found" ,
                                typeParameterArr ,
                                //paramArr
                                Param.param(
                                        //description
                                        "Ergebnis-Liste Suche" , 
                                        //clazz
                                        new ClassWithTypeParams(
                                                List.class ,
                                                new TypeVariable(
                                                        "type-Parameter, Element-Typ der Ergebnisliste der Suche" ,
                                                        "E" ) ) ,
                                        //name
                                        "resultList" ) ) ,
                        new EitherVariant(
                                //description
                                "ergebnislose Suche" ,
                                //name
                                "NotFound" ,
                                typeParameterArr ) ,
                        new EitherVariant(
                                //description
                                "fehlerhafte Suche" ,
                                //name
                                "Error" ,
                                typeParameterArr ,
                                //paramArr
                                Param.param(
                                        //description
                                        "aufgetretene Exception" , 
                                        //clazz
                                        new ClassWithTypeParams(
                                                Exception.class ) ,
                                        //name
                                        "exc" ) ,
                                Param.param(
                                        //description
                                        "Fehler-Meldungs-String" , 
                                        //clazz
                                        new ClassWithTypeParams(
                                                String.class ) ,
                                        //name
                                        "errMsgStr" ) ) ,
                        new EitherVariant(
                                //description
                                "fehlerhafte Such-Parameter" ,
                                //name
                                "ValidationError" ,
                                typeParameterArr ,
                                //paramArr
                                Param.param(
                                        //description
                                        "aufgetretene Validierungs-Exception" , 
                                        //clazz
                                        new ClassWithTypeParams(
                                                ValidationException.class ) ,
                                        //name
                                        "validationExc" ) ,
                                Param.param(
                                        //description
                                        "Fehler-Meldungs-String" , 
                                        //clazz
                                        new ClassWithTypeParams(
                                                String.class ) ,
                                        //name
                                        "errMsgStr" ) ) );

        EitherGenerator.generate(
                PROJECT_SRC_FOLDER_STR ,
                DESTINATION_PACKAGE_STR ,
                definition );
    }

}
