package option_either.generator.example.constructor_switch;

import java.util.Arrays;

import junit.framework.Assert;
import option_either.common.exception.ValidationException;
import option_either.generator.example.constructor_switch.generated.ExampleEitherWithConstructor;

import org.junit.Test;

/**
 * JUnit4-Testcase fr {@link ExampleEitherWithConstructor}.
 * 
 * @author Heiner K&uuml;cker
 */
public class ExampleEitherWithConstructorTest
{

    /**
     * Test method for {@link ExampleEitherWithConstructor#found}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testFound()
    {
        final ExampleEitherWithConstructor<String> sucheResult =
                ExampleEitherWithConstructor.found(
                        //resultList
                        Arrays.asList(
                                "suche war ok" ) );

        final ExampleEitherWithConstructor<String>.Either<String> either =
                sucheResult.new Either<String>()
                {
                    @Override
                    public String found(
                            final java.util.List<String> resultList )
                    {
                        Assert.assertEquals(
                                //expected
                                "suche war ok" ,
                                //actual
                                resultList.get( 0 ) );

                        return "method found called";
                    }
        
                    @Override
                    public String notFound()
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String validationError(
                            final option_either.common.exception.ValidationException validationExc,
                            final String msgStr )
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String error(
                            final java.lang.Exception exc,
                            final String msgStr )
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
                };

        final String actual =
                either.result;

        Assert.assertEquals(
                "method found called" ,
                actual );
    }

    /**
     * Test method for {@link ExampleEitherWithConstructor#notFound}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testNotFound()
    {
        final ExampleEitherWithConstructor<String> sucheResult =
                ExampleEitherWithConstructor.notFound();
        
        final ExampleEitherWithConstructor<String>.Either<String> either =
                sucheResult.new Either<String>()
                {
                    @Override
                    public String found(
                            final java.util.List<String> resultList )
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String notFound()
                    {
                        return "method notFound called";
                    }
        
                    @Override
                    public String validationError(
                            final option_either.common.exception.ValidationException validationExc,
                            final String msgStr )
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String error(
                            final java.lang.Exception exc,
                            final String msgStr )
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
                };

        final String actual =
                either.result;

        Assert.assertEquals(
                "method notFound called" ,
                actual );
    }

    /**
     * Test method for {@link ExampleEitherWithConstructor#validationError}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testValidationError()
    {
        final String msgStr = "Parameter hat falsches Format";

        final ExampleEitherWithConstructor<String> sucheResult =
                ExampleEitherWithConstructor.validationError(
                        new ValidationException(
                                msgStr ) ,
                        msgStr );
        
        final ExampleEitherWithConstructor<String>.Either<String> either =
                sucheResult.new Either<String>()
                {
                    @Override
                    public String found(
                            final java.util.List<String> resultList )
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String notFound()
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String validationError(
                            final option_either.common.exception.ValidationException validationExc,
                            final String msgStr )
                    {
                        Assert.assertEquals(
                                //expected
                                "Parameter hat falsches Format" ,
                                //actual
                                validationExc.getMessage() );

                        Assert.assertEquals(
                                //expected
                                "Parameter hat falsches Format" ,
                                //actual
                                msgStr );

                        return "method validationError called";
                    }
        
                    @Override
                    public String error(
                            final java.lang.Exception exc,
                            final String msgStr )
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
                };

        final String actual =
                either.result;

        Assert.assertEquals(
                "method validationError called" ,
                actual );
    }

    /**
     * Test method for {@link ExampleEitherWithConstructor#error}.
     */
    @SuppressWarnings( "static-method" )
    @Test
    public final void testError()
    {
        final String msgStr = "Suche fehlgeschlagen";

        final ExampleEitherWithConstructor<String> sucheResult =
                ExampleEitherWithConstructor.error(
                        new Exception(
                                msgStr ) ,
                        msgStr );
        
        final ExampleEitherWithConstructor<String>.Either<String> either =
                sucheResult.new Either<String>()
                {
                    @Override
                    public String found(
                            final java.util.List<String> resultList )
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String notFound()
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String validationError(
                            final option_either.common.exception.ValidationException validationExc,
                            final String msgStr )
                    {
                        Assert.fail( "wrong call" );
                        return null;
                    }
        
                    @Override
                    public String error(
                            final java.lang.Exception exc,
                            final String msgStr )
                    {
                        Assert.assertEquals(
                                //expected
                                "Suche fehlgeschlagen" ,
                                //actual
                                exc.getMessage() );

                        Assert.assertEquals(
                                //expected
                                "Suche fehlgeschlagen" ,
                                //actual
                                msgStr );

                        return "method error called";
                    }
                };

        final String actual =
                either.result;

        Assert.assertEquals(
                "method error called" ,
                actual );
    }

}
