package option_either.generator.example.constructor_switch.generated;

/**
 * Generierte Either-Klasse fr
 * Beispiel for Either-Code-Generator.
 * 
 * @param <E> Type-Parameter, Element-Typ der Ergebnisliste der Suche
 *
 * Generated by {@link option_either.generator.example.constructor_switch.ExampleEitherWithConstructorGenerator}
 */
abstract public class ExampleEitherWithConstructor<E> {

	/**
	 * Zu implementierende visit-Methode nur fr interne Verwendung.
	 *
	 * @param eitherVisitor Visitor
	 * @return frei vom Anwendungsprogrammierer verwendbar
	 */
	abstract protected Object visit(Either<?> eitherVisitor);

	/**
	 * Varianten-Klasse fr erfolgreiche Suche.
	 * 
	 * @param <E> Type-Parameter, Element-Typ der Ergebnisliste der Suche
	 */
	private static final class Found<E> extends ExampleEitherWithConstructor<E> {

		/**
		 * Ergebnis-Liste Suche	
		 */
		private final java.util.List<E> resultList;

		/**
		 * Konstruktor private,
		 * bitte Factory-Methode 
		 * {@link #found}
		 * benutzen.
		 *
		 * @param resultList Ergebnis-Liste Suche
		 */
		private Found(
				final java.util.List<E> resultList) {
			this.resultList=resultList;
		}

		/**
		 * @see ExampleEitherWithConstructor#visit
		 */
		@Override
		protected Object visit( final Either<?> eitherVisitor ) {
			return eitherVisitor.found(this.resultList);
		}

		/**
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() {
			return this.getClass().getSimpleName() +
					"[" +
					"resultList=" + this.resultList +
					"]";
		}

	}

	/**
	 * Varianten-Klasse fr ergebnislose Suche.
	 * 
	 * @param <E> Type-Parameter, Element-Typ der Ergebnisliste der Suche
	 */
	private static final class NotFound<E> extends ExampleEitherWithConstructor<E> {

		/**
		 * Konstruktor private,
		 * bitte Factory-Methode 
		 * {@link #notFound}
		 * benutzen.
		 *
		 */
		private NotFound() {
		}

		/**
		 * @see ExampleEitherWithConstructor#visit
		 */
		@Override
		protected Object visit( final Either<?> eitherVisitor ) {
			return eitherVisitor.notFound();
		}

		/**
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() {
			return this.getClass().getSimpleName();
		}

	}

	/**
	 * Varianten-Klasse fr fehlerhafte Suche.
	 * 
	 * @param <E> Type-Parameter, Element-Typ der Ergebnisliste der Suche
	 */
	private static final class Error<E> extends ExampleEitherWithConstructor<E> {

		/**
		 * aufgetretene Exception	
		 */
		private final java.lang.Exception exc;

		/**
		 * Fehler-Meldungs-String	
		 */
		private final java.lang.String errMsgStr;

		/**
		 * Konstruktor private,
		 * bitte Factory-Methode 
		 * {@link #error}
		 * benutzen.
		 *
		 * @param exc aufgetretene Exception
		 * @param errMsgStr Fehler-Meldungs-String
		 */
		private Error(
				final java.lang.Exception exc,
				final java.lang.String errMsgStr) {
			this.exc=exc;
			this.errMsgStr=errMsgStr;
		}

		/**
		 * @see ExampleEitherWithConstructor#visit
		 */
		@Override
		protected Object visit( final Either<?> eitherVisitor ) {
			return eitherVisitor.error(this.exc, this.errMsgStr);
		}

		/**
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() {
			return this.getClass().getSimpleName() +
					"[" +
					"exc=" + this.exc + ", " +
					"errMsgStr=" + this.errMsgStr +
					"]";
		}

	}

	/**
	 * Varianten-Klasse fr fehlerhafte Such-Parameter.
	 * 
	 * @param <E> Type-Parameter, Element-Typ der Ergebnisliste der Suche
	 */
	private static final class ValidationError<E> extends ExampleEitherWithConstructor<E> {

		/**
		 * aufgetretene Validierungs-Exception	
		 */
		private final option_either.common.exception.ValidationException validationExc;

		/**
		 * Fehler-Meldungs-String	
		 */
		private final java.lang.String errMsgStr;

		/**
		 * Konstruktor private,
		 * bitte Factory-Methode 
		 * {@link #validationError}
		 * benutzen.
		 *
		 * @param validationExc aufgetretene Validierungs-Exception
		 * @param errMsgStr Fehler-Meldungs-String
		 */
		private ValidationError(
				final option_either.common.exception.ValidationException validationExc,
				final java.lang.String errMsgStr) {
			this.validationExc=validationExc;
			this.errMsgStr=errMsgStr;
		}

		/**
		 * @see ExampleEitherWithConstructor#visit
		 */
		@Override
		protected Object visit( final Either<?> eitherVisitor ) {
			return eitherVisitor.validationError(this.validationExc, this.errMsgStr);
		}

		/**
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() {
			return this.getClass().getSimpleName() +
					"[" +
					"validationExc=" + this.validationExc + ", " +
					"errMsgStr=" + this.errMsgStr +
					"]";
		}

	}

	/**
	 * Abstrakte Either-Klasse.
	 * 
	 * Diese Klasse muss implementiert werden,
	 * um die konkrete Ausprgung des/der
	 * ExampleEitherWithConstructor
	 * und die mitgefhrten Werte zu erhalten.
	 * 
	 * @param <R> Type-Parameter, Type-Parameter, Typ der Member {@link #result}
	 */
	abstract public class Either<R> {

		/**
		 * Zu implementierende Methode fr 
		 * erfolgreiche Suche.
		 * 
		 * @param resultList Ergebnis-Liste Suche
		 * @return frei vom Anwendungsprogrammierer verwendbar
		 */
		abstract public R found(
				final java.util.List<E> resultList);

		/**
		 * Zu implementierende Methode fr 
		 * ergebnislose Suche.
		 * 
		 * @return frei vom Anwendungsprogrammierer verwendbar
		 */
		abstract public R notFound();

		/**
		 * Zu implementierende Methode fr 
		 * fehlerhafte Suche.
		 * 
		 * @param exc aufgetretene Exception
		 * @param errMsgStr Fehler-Meldungs-String
		 * @return frei vom Anwendungsprogrammierer verwendbar
		 */
		abstract public R error(
				final java.lang.Exception exc,
				final java.lang.String errMsgStr);

		/**
		 * Zu implementierende Methode fr 
		 * fehlerhafte Such-Parameter.
		 * 
		 * @param validationExc aufgetretene Validierungs-Exception
		 * @param errMsgStr Fehler-Meldungs-String
		 * @return frei vom Anwendungsprogrammierer verwendbar
		 */
		abstract public R validationError(
				final option_either.common.exception.ValidationException validationExc,
				final java.lang.String errMsgStr);

		/**
		 * Result-Member fr Either
		 */
		public final R result;

		/**
		 * Konstruktor mit Aufruf der Methode fr konkrete Variante des/der
		 * Beispiel for Either-Code-Generator.
		 */
		@SuppressWarnings( "unchecked" )
		public Either() {
			this.result = (R) visit( this );
		}

		/**
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() {
			return this.getClass().getSimpleName() +
					"[" +
					"ExampleEitherWithConstructor.this=" + ExampleEitherWithConstructor.this +", " +
					"result=" + this.result +
					"]";
		}

	}

	/**
	 * ffentliche Factory-Methode.
	 *
	 * @return erfolgreiche Suche
	 * @param resultList Ergebnis-Liste Suche
	 */
	public static <E> Found<E> found(
				final java.util.List<E> resultList) {
		return new Found<E>(
				resultList);
	}

	/**
	 * ffentliche Factory-Methode.
	 *
	 * @return ergebnislose Suche
	 */
	public static <E> NotFound<E> notFound() {
		return new NotFound<E>();
	}

	/**
	 * ffentliche Factory-Methode.
	 *
	 * @return fehlerhafte Suche
	 * @param exc aufgetretene Exception
	 * @param errMsgStr Fehler-Meldungs-String
	 */
	public static <E> Error<E> error(
				final java.lang.Exception exc,
				final java.lang.String errMsgStr) {
		return new Error<E>(
				exc,
				errMsgStr);
	}

	/**
	 * ffentliche Factory-Methode.
	 *
	 * @return fehlerhafte Such-Parameter
	 * @param validationExc aufgetretene Validierungs-Exception
	 * @param errMsgStr Fehler-Meldungs-String
	 */
	public static <E> ValidationError<E> validationError(
				final option_either.common.exception.ValidationException validationExc,
				final java.lang.String errMsgStr) {
		return new ValidationError<E>(
				validationExc,
				errMsgStr);
	}

}
