package option_either.generator.example.execute_method_switch;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.util.Arrays;
import java.util.List;

import option_either.common.exception.ValidationException;
import option_either.generator.example.execute_method_switch.generated.ExampleEitherWithExecuteMethod;

/**
 * Klasse zum Ausfhren einer Suche
 * als Beispiel fr die Nutzung
 * des Option-/Either-Style in Java.
 * 
 * @author Heiner K&uuml;cker
 */
public final class ExampleEitherWithExecuteMethodSucheService
{
    /**
     * Logger
     */
    // TODO private static final Logger LOGGER = Logger.getLogger( ExampleEitherWithExecuteMethodSucheService.class );

    /**
     * Konstruktor private,
     * Instanziieren nicht erlaubt.
     */
    private ExampleEitherWithExecuteMethodSucheService()
    {
        super();
    }

    /**
     * Daten Suchen.
     * 
     * @param suchStr Suchkriterium als String
     * @return Ergebnisdaten im Option-/Either-Style
     */
    public static ExampleEitherWithExecuteMethod<String> search(
            final String suchStr )
    {
        try
        {
            return 
            		innerSuche(
                            suchStr );
        }
        catch ( final ValidationException vexc )
        {
            // TODO LOGGER.error(
        	System.out.println(
                    "Fehler in Suche " +
                    vexc );

            return ExampleEitherWithExecuteMethod.validationError(
                    vexc ,
                    // Ausgabe Fehlermeldung ohne StackTrace
                    //msgStr
                    vexc.getMessage() );
        }
        catch ( final Exception exc )
        {
            // TODO LOGGER.error(
        	System.out.println(
                    "Fehler in Suche " +
                    exc );

            final String stackTraceStr =
                    stackTraceToStr(
                            exc );

            return ExampleEitherWithExecuteMethod.error(
                    exc ,
                    // Ausgabe Fehlermeldung mit StackTrace
                    //msgStr
                    exc.getMessage() + "\n" +
                    stackTraceStr );
        }
    }

    /**
     * @param exc
     * @return StackTrace als String
     */
    private static String stackTraceToStr(
            final Exception exc )
    {
        final ByteArrayOutputStream baos =
                new ByteArrayOutputStream();

        final PrintStream printStream =
                new PrintStream(
                        baos );

        exc.printStackTrace(
                printStream );

        printStream.flush();
        printStream.close();

        final String stackTraceStr =
                new String(
                        baos.toByteArray() );

        return stackTraceStr;
    }

    /**
     * @param suchStr
     * @return {@link ExampleEitherWithExecuteMethod}-Objekt
     * @throws ValidationException
     */
    private static ExampleEitherWithExecuteMethod<String> innerSuche(
            final String suchStr )
    throws ValidationException
    {
        final List<String> sucheErgebnisFakeStrList =
                Arrays.asList(
                        // TODO hier Suche mocken
                        "mock" );

        if ( sucheErgebnisFakeStrList == null )
        {
            return ExampleEitherWithExecuteMethod.notFound();
        }

        return
                ExampleEitherWithExecuteMethod.found(
                        sucheErgebnisFakeStrList );
    }

}
