package option_either.util;


/**
 * Puffer-Klasse fr beliebig
 * viele Teil-Strings, die
 * unter Verwendung eines
 * bestimmten Trenn-Strings
 * {@link #separator}
 * zusammengefgt werden
 * sollen.
 * 
 * @author Heiner K&uuml;cker
 */
public class SeparatedStrBuff
{
    /**
     * Trenn-String der einzelnen
     * Teil-Strings
     */
    public final String separator;
    
    /**
     * innerer Puffer
     */
    private final StringBuilder buffer =
        new StringBuilder();
    
    /**
     * Merker, ob der erste Teil-String
     * bereits eingefgt wurde.
     */
    private boolean isNotFirst = false;

    /**
     * Konstruktor.
     *
     * @param separator Trenn-String
     */
    public SeparatedStrBuff( 
            final String separator )
    {
        this.separator = separator;
    }
    
    /**
     * Teil-String hinzufgen
     * @param str Teil-String
     */
    public final void add(
            final String str )
    {
        if ( this.isNotFirst )
        {
            this.buffer.append( this.separator );
        }
        else
        {
            this.isNotFirst = true;
        }
        this.buffer.append( str );
    }
    
    /**
     * @param c
     */
    public void add(
            final char c )
    {
        add( String.valueOf( c ) );
    }

    /**
     * Add-All.
     * 
     * @param strArr hinzuzufgende Strings vararg
     * @return self for fluent interface
     */
    public final SeparatedStrBuff addAll(
            final String... strArr )
    {
        for ( String str : strArr )
        {
            add( str );
        }
        return this;
    }

    /**
     * Add-All.
     * 
     * @param strItbl hinzuzufgende Strings als {@link Iterable}
     * @return self for fluent interface
     */
    public final SeparatedStrBuff addAll(
            final /*Collection*/Iterable<String> strItbl )
    {
        for ( String str : strItbl )
        {
            add( str );
        }
        return this;
    }

    /**
     * @return zusammengefgter String
     * @see StringBuilder#toString()
     */
    @Override
    public final String toString()
    {
        return this.buffer.toString();
    }
    
    /**
     * @return ob der Puffer leer ist
     */
    public final boolean isEmpty()
    {
        return this.buffer.length() == 0;
    }

    /**
     * @return ob der Puffer nicht leer ist
     */
    public final boolean isNotEmpty()
    {
        return ! isEmpty();
    }

}
